import os
import dj_database_url  # type: ignore

# django settings for tests
SECRET_KEY = "test"
INSTALLED_APPS = (
    "django.contrib.contenttypes",
    "openstates.data",
)
MIDDLEWARE_CLASSES = ()
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": "test",
        "USER": "test",
        "PASSWORD": "test",
        "HOST": os.environ.get("POSTGRES_HOST", "localhost"),
        "PORT": os.environ.get("POSTGRES_PORT", "5432"),
    }
}
TIME_ZONE = "UTC"
DATABASE_URL = os.environ.get("OVERRIDE_DATABASE_URL")
if DATABASE_URL:
    DATABASES = {"default": dj_database_url.parse(DATABASE_URL)}



from ..models import State, Chamber, District, simple_numbered_districts

ND = State(
    name="North Dakota",
    abbr="ND",
    capital="Bismarck",
    capital_tz="America/North_Dakota/Center",
    fips="38",
    unicameral=False,
    legislature_name="North Dakota Legislative Assembly",
    legislature_organization_id="ocd-organization/aa301a79-0ad8-4905-818c-0cd5f1ce5946",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/5ba1c008-42e8-55ad-864e-e95b318d7ce4",
    division_id="ocd-division/country:us/state:nd",
    jurisdiction_id="ocd-jurisdiction/country:us/state:nd/government",
    url="http://www.legis.nd.gov/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/b9f9b32c-a2cc-4cca-941f-862aecc30617",
        num_seats=94,
        title="Representative",
        districts=[
            District("1", "lower", "ocd-division/country:us/state:nd/sldl:1", 2),
            District("2", "lower", "ocd-division/country:us/state:nd/sldl:2", 2),
            District("3", "lower", "ocd-division/country:us/state:nd/sldl:3", 2),
            District("4A", "lower", "ocd-division/country:us/state:nd/sldl:4a", 1),
            District("4B", "lower", "ocd-division/country:us/state:nd/sldl:4b", 1),
            District("5", "lower", "ocd-division/country:us/state:nd/sldl:5", 2),
            District("6", "lower", "ocd-division/country:us/state:nd/sldl:6", 2),
            District("7", "lower", "ocd-division/country:us/state:nd/sldl:7", 2),
            District("8", "lower", "ocd-division/country:us/state:nd/sldl:8", 2),
            District("9A", "lower", "ocd-division/country:us/state:nd/sldl:9a", 1),
            District("9B", "lower", "ocd-division/country:us/state:nd/sldl:9b", 1),
            District("10", "lower", "ocd-division/country:us/state:nd/sldl:10", 2),
            District("11", "lower", "ocd-division/country:us/state:nd/sldl:11", 2),
            District("12", "lower", "ocd-division/country:us/state:nd/sldl:12", 2),
            District("13", "lower", "ocd-division/country:us/state:nd/sldl:13", 2),
            District("14", "lower", "ocd-division/country:us/state:nd/sldl:14", 2),
            District("15", "lower", "ocd-division/country:us/state:nd/sldl:15", 2),
            District("16", "lower", "ocd-division/country:us/state:nd/sldl:16", 2),
            District("17", "lower", "ocd-division/country:us/state:nd/sldl:17", 2),
            District("18", "lower", "ocd-division/country:us/state:nd/sldl:18", 2),
            District("19", "lower", "ocd-division/country:us/state:nd/sldl:19", 2),
            District("20", "lower", "ocd-division/country:us/state:nd/sldl:20", 2),
            District("21", "lower", "ocd-division/country:us/state:nd/sldl:21", 2),
            District("22", "lower", "ocd-division/country:us/state:nd/sldl:22", 2),
            District("23", "lower", "ocd-division/country:us/state:nd/sldl:23", 2),
            District("24", "lower", "ocd-division/country:us/state:nd/sldl:24", 2),
            District("25", "lower", "ocd-division/country:us/state:nd/sldl:25", 2),
            District("26", "lower", "ocd-division/country:us/state:nd/sldl:26", 2),
            District("27", "lower", "ocd-division/country:us/state:nd/sldl:27", 2),
            District("28", "lower", "ocd-division/country:us/state:nd/sldl:28", 2),
            District("29", "lower", "ocd-division/country:us/state:nd/sldl:29", 2),
            District("30", "lower", "ocd-division/country:us/state:nd/sldl:30", 2),
            District("31", "lower", "ocd-division/country:us/state:nd/sldl:31", 2),
            District("32", "lower", "ocd-division/country:us/state:nd/sldl:32", 2),
            District("33", "lower", "ocd-division/country:us/state:nd/sldl:33", 2),
            District("34", "lower", "ocd-division/country:us/state:nd/sldl:34", 2),
            District("35", "lower", "ocd-division/country:us/state:nd/sldl:35", 2),
            District("36", "lower", "ocd-division/country:us/state:nd/sldl:36", 2),
            District("37", "lower", "ocd-division/country:us/state:nd/sldl:37", 2),
            District("38", "lower", "ocd-division/country:us/state:nd/sldl:38", 2),
            District("39", "lower", "ocd-division/country:us/state:nd/sldl:39", 2),
            District("40", "lower", "ocd-division/country:us/state:nd/sldl:40", 2),
            District("41", "lower", "ocd-division/country:us/state:nd/sldl:41", 2),
            District("42", "lower", "ocd-division/country:us/state:nd/sldl:42", 2),
            District("43", "lower", "ocd-division/country:us/state:nd/sldl:43", 2),
            District("44", "lower", "ocd-division/country:us/state:nd/sldl:44", 2),
            District("45", "lower", "ocd-division/country:us/state:nd/sldl:45", 2),
            District("46", "lower", "ocd-division/country:us/state:nd/sldl:46", 2),
            District("47", "lower", "ocd-division/country:us/state:nd/sldl:47", 2),
        ],
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/79645ed7-6baa-4842-ac97-09b444c4020b",
        num_seats=47,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:nd", "upper", 47
        ),
    ),
)



from ..models import State, Chamber, District, simple_numbered_districts

MD = State(
    name="Maryland",
    abbr="MD",
    capital="Annapolis",
    capital_tz="America/New_York",
    fips="24",
    unicameral=False,
    legislature_name="Maryland General Assembly",
    legislature_organization_id="ocd-organization/15f3b217-1d89-40dd-ab4e-d5ed2ffe5322",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/8af93cc2-fc5f-48b9-abc9-53b7a56b2beb",
    division_id="ocd-division/country:us/state:md",
    jurisdiction_id="ocd-jurisdiction/country:us/state:md/government",
    url="http://mgaleg.maryland.gov/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/bd8388d3-8fcd-4e28-9c1b-f23f0e376544",
        num_seats=141,
        title="Delegate",
        districts=[
            District("1A", "lower", "ocd-division/country:us/state:md/sldl:1a", 1),
            District("1B", "lower", "ocd-division/country:us/state:md/sldl:1b", 1),
            District("1C", "lower", "ocd-division/country:us/state:md/sldl:1c", 1),
            District("2A", "lower", "ocd-division/country:us/state:md/sldl:2a", 2),
            District("2B", "lower", "ocd-division/country:us/state:md/sldl:2b", 1),
            District("3", "lower", "ocd-division/country:us/state:md/sldl:3", 3),
            District("4", "lower", "ocd-division/country:us/state:md/sldl:4", 3),
            District("5", "lower", "ocd-division/country:us/state:md/sldl:5", 3),
            District("6", "lower", "ocd-division/country:us/state:md/sldl:6", 3),
            District("7A", "lower", "ocd-division/country:us/state:md/sldl:7a", 2),
            District("7B", "lower", "ocd-division/country:us/state:md/sldl:7b", 1),
            District("8", "lower", "ocd-division/country:us/state:md/sldl:8", 3),
            District("9A", "lower", "ocd-division/country:us/state:md/sldl:9a", 2),
            District("9B", "lower", "ocd-division/country:us/state:md/sldl:9b", 1),
            District("10", "lower", "ocd-division/country:us/state:md/sldl:10", 3),
            District("11A", "lower", "ocd-division/country:us/state:md/sldl:11a", 1),
            District("11B", "lower", "ocd-division/country:us/state:md/sldl:11b", 2),
            District("12A", "lower", "ocd-division/country:us/state:md/sldl:12a", 2),
            District("12B", "lower", "ocd-division/country:us/state:md/sldl:12b", 1),
            District("13", "lower", "ocd-division/country:us/state:md/sldl:13", 3),
            District("14", "lower", "ocd-division/country:us/state:md/sldl:14", 3),
            District("15", "lower", "ocd-division/country:us/state:md/sldl:15", 3),
            District("16", "lower", "ocd-division/country:us/state:md/sldl:16", 3),
            District("17", "lower", "ocd-division/country:us/state:md/sldl:17", 3),
            District("18", "lower", "ocd-division/country:us/state:md/sldl:18", 3),
            District("19", "lower", "ocd-division/country:us/state:md/sldl:19", 3),
            District("20", "lower", "ocd-division/country:us/state:md/sldl:20", 3),
            District("21", "lower", "ocd-division/country:us/state:md/sldl:21", 3),
            District("22", "lower", "ocd-division/country:us/state:md/sldl:22", 3),
            District("23", "lower", "ocd-division/country:us/state:md/sldl:23", 3),
            District("24", "lower", "ocd-division/country:us/state:md/sldl:24", 3),
            District("25", "lower", "ocd-division/country:us/state:md/sldl:25", 3),
            District("26", "lower", "ocd-division/country:us/state:md/sldl:26", 3),
            District("27A", "lower", "ocd-division/country:us/state:md/sldl:27a", 1),
            District("27B", "lower", "ocd-division/country:us/state:md/sldl:27b", 1),
            District("27C", "lower", "ocd-division/country:us/state:md/sldl:27c", 1),
            District("28", "lower", "ocd-division/country:us/state:md/sldl:28", 3),
            District("29A", "lower", "ocd-division/country:us/state:md/sldl:29a", 1),
            District("29B", "lower", "ocd-division/country:us/state:md/sldl:29b", 1),
            District("29C", "lower", "ocd-division/country:us/state:md/sldl:29c", 1),
            District("30A", "lower", "ocd-division/country:us/state:md/sldl:30a", 2),
            District("30B", "lower", "ocd-division/country:us/state:md/sldl:30b", 1),
            District("31", "lower", "ocd-division/country:us/state:md/sldl:31", 3),
            District("32", "lower", "ocd-division/country:us/state:md/sldl:32", 3),
            District("33A", "lower", "ocd-division/country:us/state:md/sldl:33a", 1),
            District("33B", "lower", "ocd-division/country:us/state:md/sldl:33b", 1),
            District("33C", "lower", "ocd-division/country:us/state:md/sldl:33c", 1),
            District("34A", "lower", "ocd-division/country:us/state:md/sldl:34a", 2),
            District("34B", "lower", "ocd-division/country:us/state:md/sldl:34b", 1),
            District("35A", "lower", "ocd-division/country:us/state:md/sldl:35a", 2),
            District("35B", "lower", "ocd-division/country:us/state:md/sldl:35b", 1),
            District("36", "lower", "ocd-division/country:us/state:md/sldl:36", 3),
            District("37A", "lower", "ocd-division/country:us/state:md/sldl:37a", 1),
            District("37B", "lower", "ocd-division/country:us/state:md/sldl:37b", 2),
            District("38A", "lower", "ocd-division/country:us/state:md/sldl:38a", 1),
            District("38B", "lower", "ocd-division/country:us/state:md/sldl:38b", 1),
            District("38C", "lower", "ocd-division/country:us/state:md/sldl:38c", 1),
            District("39", "lower", "ocd-division/country:us/state:md/sldl:39", 3),
            District("40", "lower", "ocd-division/country:us/state:md/sldl:40", 3),
            District("41", "lower", "ocd-division/country:us/state:md/sldl:41", 3),
            District("42A", "lower", "ocd-division/country:us/state:md/sldl:42a", 1),
            District("42B", "lower", "ocd-division/country:us/state:md/sldl:42b", 1),
            District("42C", "lower", "ocd-division/country:us/state:md/sldl:42c", 1),
            District("43A", "lower", "ocd-division/country:us/state:md/sldl:43a", 2),
            District("43B", "lower", "ocd-division/country:us/state:md/sldl:43b", 1),
            District("44A", "lower", "ocd-division/country:us/state:md/sldl:44a", 1),
            District("44B", "lower", "ocd-division/country:us/state:md/sldl:44b", 2),
            District("45", "lower", "ocd-division/country:us/state:md/sldl:45", 3),
            District("46", "lower", "ocd-division/country:us/state:md/sldl:46", 3),
            District("47A", "lower", "ocd-division/country:us/state:md/sldl:47a", 2),
            District("47B", "lower", "ocd-division/country:us/state:md/sldl:47b", 1),
        ],
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/6328292e-c35e-4187-903c-75829a17d5d8",
        num_seats=47,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:md", "upper", 47
        ),
    ),
)



from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from openstates.data.admin.organization import OrganizationInline

from .. import models
from .base import ModelAdmin, ReadOnlyTabularInline


class JurisdictionInline(ReadOnlyTabularInline):
    model = models.Jurisdiction
    list_display = ("name", "id")
    readonly_fields = fields = (
        "get_id",
        "name",
        "classification",
        "extras",
        "url",
    )
    ordering = ("id",)

    def get_id(self, jurisdiction):
        admin_url = reverse(
            "admin:data_jurisdiction_change", args=(jurisdiction.pk,)
        )
        tmpl = u'<a href="%s">%s</a>'
        return format_html(tmpl % (admin_url, jurisdiction.name))

    get_id.short_description = "ID"
    get_id.allow_tags = True
    get_id.admin_order_field = "jurisdiction__name"


@admin.register(models.Division)
class DivisionAdmin(ModelAdmin):
    list_display = ("name", "id")
    search_fields = list_display
    fields = readonly_fields = ("id", "name", "redirect", "country")
    ordering = ("id",)
    inlines = [JurisdictionInline]


class LegislativeSessionInline(ReadOnlyTabularInline):
    model = models.LegislativeSession
    readonly_fields = ("identifier", "get_name", "classification", "start_date", "end_date")
    fields = readonly_fields
    ordering = ("-identifier",)

    def get_name(self, legislative_session):
        admin_url = reverse(
            "admin:data_legislativesession_change", args=(legislative_session.pk,)
        )
        tmpl = u'<a href="%s">%s</a>'
        return format_html(tmpl % (admin_url, legislative_session.name))

    get_name.short_description = "NAME"
    get_name.allow_tags = True


class BillInline(ReadOnlyTabularInline):
    model = models.Bill
    readonly_fields = fields = (
        "get_identifier",
        "classification",
        "from_organization",
        "title",
        "latest_action_date",
        "subject",
    )
    ordering = ("identifier",)

    def get_identifier(self, bill):
        admin_url = reverse(
            "admin:data_bill_change", args=(bill.id,)
        )
        tmpl = u'<a href="%s">%s</a>'
        return format_html(tmpl % (admin_url, bill.identifier))


@admin.register(models.LegislativeSession)
class LegislativeSessionAdmin(ModelAdmin):
    readonly_fields = ("jurisdiction", "identifier", "name", "active", "classification", "start_date", "end_date")
    inlines = [BillInline]


@admin.register(models.Jurisdiction)
class JurisdictionAdmin(ModelAdmin):
    list_display = ("name", "id", "classification")
    readonly_fields = fields = (
        "id",
        "name",
        "division",
        "classification",
        "extras",
        "url",
    )
    ordering = ("-classification", "id",)
    search_fields = ("id", "name")
    inlines = [LegislativeSessionInline, OrganizationInline]



from ..models import State, Chamber, simple_numbered_districts

AL = State(
    name="Alabama",
    abbr="AL",
    capital="Montgomery",
    capital_tz="America/Chicago",
    fips="01",
    unicameral=False,
    legislature_name="Alabama Legislature",
    legislature_organization_id="ocd-organization/8de9200f-a2e2-4f92-9950-e6c63f265248",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/f36ec162-a87e-57ab-976b-98b047a854f4",
    division_id="ocd-division/country:us/state:al",
    jurisdiction_id="ocd-jurisdiction/country:us/state:al/government",
    url="http://www.legislature.state.al.us/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/74ad748b-f0a1-408f-9aa6-3319a868fd55",
        num_seats=105,
        title="Representative",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:al", "lower", 105
        ),
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/ddf820b5-5246-46b3-a807-99b5914ad39f",
        num_seats=35,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:al", "upper", 35
        ),
    ),
)



from django.urls import reverse
from django.contrib import admin
from django.utils.html import format_html

from openstates.data.admin.person import MembershipInline

from .. import models
from .base import ModelAdmin, ReadOnlyTabularInline


class PostInline(admin.TabularInline):
    """a read-only inline for posts here, with links to the real thing"""

    model = models.Post
    extra = 0
    fields = readonly_fields = ("get_label", "role")
    ordering = ("label",)
    can_delete = False

    def get_label(self, post):
        admin_url = reverse(
            "admin:data_post_change", args=(post.pk,)
        )
        tmpl = u'<a href="%s">%s</a>'
        return format_html(tmpl % (admin_url, post.label))

    get_label.short_description = "Label"

    def has_add_permission(self, request, obj=None):
        return False


class OrganizationInline(ReadOnlyTabularInline):
    """a read-only inline for posts here, with links to the real thing"""

    model = models.Organization
    fields = readonly_fields = ("get_name", "jurisdiction", "classification")
    ordering = ("-classification", "name")

    def get_name(self, organization):
        admin_url = reverse(
            "admin:data_organization_change", args=(organization.pk,)
        )
        tmpl = u'<a href="%s">%s</a>'
        return format_html(tmpl % (admin_url, organization.name))

    get_name.short_description = "ID"
    get_name.allow_tags = True
    get_name.admin_order_field = "organization__name"


class OrgMembershipInline(ReadOnlyTabularInline):
    model = models.Membership
    fk_name = "organization"
    readonly_fields = ("id", "person", "post", "role", "start_date")
    fields = readonly_fields + ("end_date",)
    extra = 0
    can_delete = False
    ordering = ("person__name", "start_date")


@admin.register(models.Post)
class PostAdmin(ModelAdmin):
    readonly_fields = fields = ("id", "division", "organization", "label", "role", "maximum_memberships", "extras")
    ordering = ("division__id", "organization", "role", "label")
    inlines = (MembershipInline,)
    list_display = ("division", "organization", "role", "label")


@admin.register(models.Organization)
class OrganizationAdmin(ModelAdmin):
    readonly_fields = (
        "id",
        "name",
        "classification",
        "parent",
        "jurisdiction",
        "extras",
    )
    fields = readonly_fields
    search_fields = ("name",)
    list_filter = ("jurisdiction__name",)

    inlines = [
        PostInline,
        OrgMembershipInline,
        OrganizationInline,
    ]

    def get_org_name(self, obj):
        parent = obj.parent
        if parent:
            return "{org} ({parent})".format(org=obj.name, parent=parent.name)
        return obj.name

    get_org_name.short_description = "Name"
    get_org_name.allow_tags = True
    get_org_name.admin_order_field = "name"

    def get_jurisdiction(self, obj):
        jurisdiction = obj.jurisdiction
        if jurisdiction:
            admin_url = reverse(
                "admin:data_jurisdiction_change", args=(jurisdiction.pk,)
            )
            tmpl = '<a href="%s">%s</a>'
            return format_html(tmpl % (admin_url, jurisdiction.name))

        return "(none)"

    get_jurisdiction.short_description = "Jurisdiction"
    get_jurisdiction.allow_tags = True
    get_jurisdiction.admin_order_field = "jurisdiction__name"

    list_select_related = ("jurisdiction",)
    list_display = ("get_org_name", "get_jurisdiction", "classification")
    ordering = ("jurisdiction__name", "-classification", "name")



import os
import django  # type: ignore
from django import conf  # type: ignore
import dj_database_url  # type: ignore


def init_django() -> None:  # pragma: no cover
    DATABASE_URL = os.environ.get(
        "DATABASE_URL", "postgis://openstates:openstates@localhost/openstates"
    )
    DATABASES = {"default": dj_database_url.parse(DATABASE_URL)}
    application_name = "os_core"
    if "OPTIONS" not in DATABASES:
        DATABASES["default"]["OPTIONS"] = {"application_name": application_name}
    else:
        DATABASES["default"]["OPTIONS"]["application_name"] = application_name

    conf.settings.configure(
        conf.global_settings,
        SECRET_KEY="not-important",
        DEBUG=False,
        INSTALLED_APPS=(
            "django.contrib.contenttypes",
            "openstates.data",
        ),
        DATABASES=DATABASES,
        TIME_ZONE="UTC",
        MIDDLEWARE_CLASSES=(),
    )
    django.setup()



from openstates.cli.people import Summarizer
from openstates.models.people import (
    Person,
    Party,
    Office,
    OtherIdentifier,
    PersonIdBlock,
)
from openstates.utils.people import ocd_uuid


def test_person_summary():
    s = Summarizer()

    people = [
        Person(
            id=ocd_uuid("person"),
            name="Jorna Corno",
            gender="F",
            image="https://example.com/image1",
            party=[Party(name="Democratic"), Party(name="Democratic", end_date="1990")],
            roles=[],
        ),
        Person(
            id=ocd_uuid("person"),
            name="Linda Under",
            gender="F",
            image="https://example.com/image2",
            party=[Party(name="Democratic"), Party(name="Progressive")],
            extras={"religion": "Zoroastrian"},
            offices=[Office(fax="123-435-9999", classification="capitol")],
            other_identifiers=[OtherIdentifier(scheme="fake", identifier="abc")],
            ids=PersonIdBlock(twitter="fake"),
            roles=[],
        ),
        Person(
            id=ocd_uuid("person"),
            name="Phil Gort",
            gender="M",
            image="https://example.com/image3",
            party=[Party(name="Republican")],
            offices=[Office(voice="123-435-9999", classification="capitol")],
            other_identifiers=[OtherIdentifier(scheme="fake", identifier="123")],
            roles=[],
        ),
    ]

    for p in people:
        s.summarize(p)

    assert s.parties == {"Republican": 1, "Democratic": 2, "Progressive": 1}
    assert s.contact_counts == {"capitol voice": 1, "capitol fax": 1}
    assert s.id_counts == {"fake": 2, "twitter": 1}
    assert s.optional_fields == {"gender": 3, "image": 3}
    assert s.extra_counts == {"religion": 1}



from ..models import State, Chamber, simple_numbered_districts

VI = State(
    name="Virgin Islands",
    abbr="VI",
    capital="Charlotte Amalie",
    capital_tz="America/Puerto_Rico",
    fips="78",
    url="https://legvi.org",
    unicameral=True,
    legislature_name="Legislature of the Virgin Islands",
    legislature_organization_id="ocd-organization/7caa0f66-1197-46d4-82e6-6a98129b0a96",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/20779ed0-c5f5-4c90-8c06-d972aed7b4b7",
    division_id="ocd-division/country:us/territory:vi",
    jurisdiction_id="ocd-jurisdiction/country:us/territory:vi/government",
    legislature=Chamber(
        chamber_type="unicameral",
        name="Legislature",
        organization_id="ocd-organization/7caa0f66-1197-46d4-82e6-6a98129b0a96",
        num_seats=15,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/territory:vi", "legislature", 15
        ),
    ),
)



# Generated by Django 3.0.5 on 2020-09-14 10:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("data", "0020_auto_20200914_1041"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="jurisdiction",
            name="feature_flags",
        ),
    ]



from openstates.utils.people.retire import retire_person
from openstates.models.people import Person, Role, Party

JID = "ocd-jurisdiction/country:us/state:nc/government"


def test_retire_person():
    person = Person(
        id="ocd-person/11110000-2222-3333-4444-555555555555",
        name="Test Person",
        party=[
            Party(name="Democratic"),
        ],
        roles=[
            Role(
                type="lower", end_date="2000-01-01", district="1", jurisdiction=JID
            ),  # leave old end date alone
            Role(
                type="upper", start_date="2018-01-01", district="2", jurisdiction=JID
            ),  # add end date
            Role(
                type="governor", end_date="2030-01-01", jurisdiction=JID
            ),  # move up future end date
        ],
    )
    person, num = retire_person(person, "2018-10-01")
    assert num == 2
    assert person.roles[0].end_date == "2000-01-01"
    assert person.roles[1].end_date == "2018-10-01"
    assert person.roles[2].end_date == "2018-10-01"

    # idempotent
    person, num = retire_person(person, "2018-11-01")
    assert num == 0



from ..exceptions import ScrapeValueError
from ..utils import _make_pseudo_id
from .base import BaseModel, cleanup_list, SourceMixin
from .bill import Bill
from .popolo import pseudo_organization
from .schemas.vote_event import schema


class VoteEvent(BaseModel, SourceMixin):
    _type = "vote_event"
    _schema = schema

    def __init__(
        self,
        *,
        motion_text,
        start_date,
        classification,
        result,
        legislative_session=None,
        identifier="",
        bill=None,
        bill_chamber=None,
        bill_action=None,
        organization=None,
        chamber=None
    ):
        super(VoteEvent, self).__init__()

        self.legislative_session = legislative_session
        self.motion_text = motion_text
        self.motion_classification = cleanup_list(classification, [])
        self.start_date = start_date
        self.result = result
        self.identifier = identifier
        self.bill_action = bill_action

        self.set_bill(bill, chamber=bill_chamber)

        if isinstance(bill, Bill) and not self.legislative_session:
            self.legislative_session = bill.legislative_session

        if not self.legislative_session:
            raise ScrapeValueError("must set legislative_session or bill")

        self.organization = pseudo_organization(organization, chamber, "legislature")
        self.votes = []
        self.counts = []

    def __str__(self):
        return "{0} - {1} - {2}".format(
            self.legislative_session, self.start_date, self.motion_text
        )

    def set_bill(self, bill_or_identifier, *, chamber=None):
        if not bill_or_identifier:
            self.bill = None
            self.bill_identifier = ""
        elif isinstance(bill_or_identifier, Bill):
            if chamber:
                raise ScrapeValueError(
                    "set_bill takes no arguments when using a `Bill` object"
                )
            self.bill = bill_or_identifier._id
            self.bill_identifier = bill_or_identifier.identifier
        else:
            if chamber is None:
                chamber = "legislature"
            kwargs = {
                "identifier": bill_or_identifier,
                "from_organization__classification": chamber,
                "legislative_session__identifier": self.legislative_session,
            }
            self.bill = _make_pseudo_id(**kwargs)
            self.bill_identifier = bill_or_identifier

    def vote(self, option, voter, *, note=""):
        self.votes.append(
            {
                "option": option,
                "voter_name": voter,
                "voter_id": _make_pseudo_id(name=voter),
                "note": note,
            }
        )

    def yes(self, name, *, id=None, note=""):
        return self.vote("yes", name, note=note)

    def no(self, name, *, id=None, note=""):
        return self.vote("no", name, note=note)

    def set_count(self, option, value):
        for co in self.counts:
            if co["option"] == option:
                co["value"] = value
                break
        else:
            self.counts.append({"option": option, "value": value})



from ..models import District, simple_numbered_districts

"""
Legacy Districts is how we represent districts that have been retired due to redistricting
which occurs every 10 years
"""

legacy_districts = {
    "id": simple_numbered_districts(
        "ocd-division/country:us/state:id",
        "lower",
        35,
        num_seats=2,
    ),
    "nd": [
        District("4", "lower", "ocd-division/country:us/state:nd/sldl:4", 2),
        District("9", "lower", "ocd-division/country:us/state:nd/sldl:9", 2),
    ],
    "ma": [
        # These Districts were Retired in 2022
        District(
            "Worcester, Hampden, Hampshire and Middlesex", "upper", "ocd-division/country:us/state:ma/sldu:worcester_hampden_hampshire_and_middlesex"
        ),
        District(
            "Berkshire, Hampshire, Franklin and Hampden", "upper", "ocd-division/country:us/state:ma/sldu:berkshire_hampshire_franklin_and_hampden"
        ),
        District("Worcester and Norfolk", "upper", "ocd-division/country:us/state:ma/sldu:worcester_and_norfolk"),
        District("Second Suffolk and Middlesex", "upper", "ocd-division/country:us/state:ma/sldu:2nd_suffolk_and_middlesex"),
        District("Second Plymouth and Bristol", "upper", "ocd-division/country:us/state:ma/sldu:2nd_plymouth_and_bristol"),
        District("Second Middlesex and Norfolk", "upper", "ocd-division/country:us/state:ma/sldu:2nd_middlesex_and_norfolk"),
        District("Second Hampden and Hampshire", "upper", "ocd-division/country:us/state:ma/sldu:2nd_hampden_and_hampshire"),
        District("Plymouth and Norfolk", "upper", "ocd-division/country:us/state:ma/sldu:plymouth_and_norfolk"),
        District("Norfolk, Bristol and Plymouth", "upper", "ocd-division/country:us/state:ma/sldu:norfolk_bristol_and_plymouth"),
        District("Norfolk, Bristol and Middlesex", "upper", "ocd-division/country:us/state:ma/sldu:norfolk_bristol_and_middlesex"),
        District("First Suffolk and Middlesex", "upper", "ocd-division/country:us/state:ma/sldu:1st_suffolk_and_middlesex"),
        District("First Plymouth and Bristol", "upper", "ocd-division/country:us/state:ma/sldu:1st_plymouth_and_bristol"),
        District("First Middlesex and Norfolk", "upper", "ocd-division/country:us/state:ma/sldu:1st_middlesex_and_norfolk"),
        District("First Hampden and Hampshire", "upper", "ocd-division/country:us/state:ma/sldu:1st_hampden_and_hampshire"),
        District("4th Berkshire", "lower", "ocd-division/country:us/state:ma/sldl:4th_berkshire"),
    ],
    "md": [
        District("2C", "lower", "ocd-division/country:us/state:md/sldl:2c"),
        District("30", "lower", "ocd-division/country:us/state:md/sldl:30"),
        District("42", "lower", "ocd-division/country:us/state:md/sldl:42"),
        District("44", "lower", "ocd-division/country:us/state:md/sldl:44"),
        District("47", "lower", "ocd-division/country:us/state:md/sldl:47"),
        District("4A", "lower", "ocd-division/country:us/state:md/sldl:4a"),
        District("4B", "lower", "ocd-division/country:us/state:md/sldl:4b"),
        District("5A", "lower", "ocd-division/country:us/state:md/sldl:5a"),
        District("5B", "lower", "ocd-division/country:us/state:md/sldl:5b"),
        District("7", "lower", "ocd-division/country:us/state:md/sldl:7", 3),
        District("3A", "lower", "ocd-division/country:us/state:md/sldl:3a", 2),
        District("3B", "lower", "ocd-division/country:us/state:md/sldl:3b"),
        District("11", "lower", "ocd-division/country:us/state:md/sldl:11", 3),
        District("12", "lower", "ocd-division/country:us/state:md/sldl:12", 3),
        District("23A", "lower", "ocd-division/country:us/state:md/sldl:23a"),
        District("23B", "lower", "ocd-division/country:us/state:md/sldl:23b", 2),
        District("31A", "lower", "ocd-division/country:us/state:md/sldl:31a"),
        District("31B", "lower", "ocd-division/country:us/state:md/sldl:31b", 2),
        District("33", "lower", "ocd-division/country:us/state:md/sldl:33", 3),
        District("43", "lower", "ocd-division/country:us/state:md/sldl:43", 3),
    ],
    "nh": [
        District("Strafford 22", "lower", "ocd-division/country:us/state:nh/sldl:strafford_22"),
        District("Strafford 23", "lower", "ocd-division/country:us/state:nh/sldl:strafford_23"),
        District("Strafford 24", "lower", "ocd-division/country:us/state:nh/sldl:strafford_24"),
        District("Strafford 25", "lower", "ocd-division/country:us/state:nh/sldl:strafford_25"),
        District("Sullivan 9", "lower", "ocd-division/country:us/state:nh/sldl:sullivan_9"),
        District("Sullivan 10", "lower", "ocd-division/country:us/state:nh/sldl:sullivan_10"),
        District("Sullivan 11", "lower", "ocd-division/country:us/state:nh/sldl:sullivan_11"),
        District("Belknap 9", "lower", 'ocd-division/country:us/state:nh/sldl:belknap_9'),
    ],
    "nv": [
        District("Capital Senatorial District", "upper", "ocd-division/country:us/state:nv/sldu:capital"),
        District("Central Nevada Senatorial District", "upper", "ocd-division/country:us/state:nv/sldu:central_nevada"),
        District("Clark County, No. 1", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_1"),
        District("Clark County, No. 2", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_2"),
        District("Clark County, No. 3", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_3"),
        District("Clark County, No. 4", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_4"),
        District("Clark County, No. 5", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_5"),
        District("Clark County, No. 6", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_6"),
        District("Clark County, No. 7", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_7"),
        District("Clark County, No. 8", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_8"),
        District("Clark County, No. 9", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_9"),
        District("Clark County, No. 10", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_10"),
        District("Clark County, No. 11", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_11"),
        District("Clark County, No. 12", "upper", "ocd-division/country:us/state:nv/sldu:clark_county_12"),
        District("Rural Nevada Senatorial District", "upper", "ocd-division/country:us/state:nv/sldu:rural_nevada"),
        District("Washoe County, No. 1", "upper", "ocd-division/country:us/state:nv/sldu:washoe_county_1"),
        District("Washoe County, No. 2", "upper", "ocd-division/country:us/state:nv/sldu:washoe_county_2"),
        District("Washoe County, No. 3", "upper", "ocd-division/country:us/state:nv/sldu:washoe_county_3"),
        District("Washoe County, No. 4", "upper", "ocd-division/country:us/state:nv/sldu:washoe_county_4"),
    ],
    "pr": [
        District("I", "upper", division_id=None),
        District("II", "upper", division_id=None),
        District("III", "upper", division_id=None),
        District("IV", "upper", division_id=None),
        District("V", "upper", division_id=None),
        District("VI", "upper", division_id=None),
        District("VII", "upper", division_id=None),
        District("VIII", "upper", division_id=None),
    ],
    "vt": [
        District("Addison-Rutland-1", "lower", "ocd-division/country:us/state:vt/sldl:addison-rutland-1"),
        District("Bennington-Rutland-1", "lower", "ocd-division/country:us/state:vt/sldl:bennington-rutland-1"),
        District("Chittenden-1-1", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-1-1"),
        District("Chittenden-1-2", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-1-2"),
        District("Chittenden-3-1", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-1"),
        District("Chittenden-3-2", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-2"),
        District("Chittenden-3-3", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-3"),
        District("Chittenden-3-4", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-4"),
        District("Chittenden-3-5", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-5"),
        District("Chittenden-3-6", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-6"),
        District("Chittenden-3-7", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-7"),
        District("Chittenden-3-8", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-8"),
        District("Chittenden-3-9", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-9"),
        District("Chittenden-3-10", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-3-10"),
        District("Chittenden-4", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-4"),
        District("Chittenden-8", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-8"),
        District("Chittenden-9", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-9"),
        District("Franklin-3", "lower", "ocd-division/country:us/state:vt/sldl:franklin-3"),
        District("Grand Isle-Chittenden-1", "lower", "ocd-division/country:us/state:vt/sldl:grand_isle-chittenden-1"),
        District("Lamoille-4", "lower", "ocd-division/country:us/state:vt/sldl:lamoille-4"),
        District("Lamoille-Washington-1", "lower", "ocd-division/country:us/state:vt/sldl:lamoille-washington-1"),
        District("Orange-Addison-1", "lower", "ocd-division/country:us/state:vt/sldl:orange-addison-1"),
        District("Orleans-Caledonia-1", "lower", "ocd-division/country:us/state:vt/sldl:orange-caledonia-1"),
        District("Rutland-1-1", "lower", "ocd-division/country:us/state:vt/sldl:rutland-1-1"),
        District("Rutland-1-2", "lower", "ocd-division/country:us/state:vt/sldl:rutland-1-2"),
        District("Rutland-7", "lower", "ocd-division/country:us/state:vt/sldl:rutland-7"),
        District("Rutland-8", "lower", "ocd-division/country:us/state:vt/sldl:rutland-8"),
        District("Washington-3-1", "lower", "ocd-division/country:us/state:vt/sldl:washington-3-1"),
        District("Washington-3-2", "lower", "ocd-division/country:us/state:vt/sldl:washington-3-2"),
        District("Washington-3-3", "lower", "ocd-division/country:us/state:vt/sldl:washington-3-3"),
        District("Washington-Chittenden-1", "lower", "ocd-division/country:us/state:vt/sldl:washington-chittenden-1"),
        District("Windham-3-1", "lower", "ocd-division/country:us/state:vt/sldl:windham-3-1"),
        District("Windham-3-2", "lower", "ocd-division/country:us/state:vt/sldl:windham-3-2"),
        District("Windham-3-3", "lower", "ocd-division/country:us/state:vt/sldl:windham-3-3"),
        District("Windham-Bennington-1", "lower", "ocd-division/country:us/state:vt/sldl:windham-bennington-1"),
        District("Windham-Bennington-Windsor-1", "lower", division_id=None),
        District("Windsor-1-1", "lower", "ocd-division/country:us/state:vt/sldl:windsor-1-1"),
        District("Windsor-1-2", "lower", "ocd-division/country:us/state:vt/sldl:windsor-1-2"),
        District("Windsor-3", "lower", "ocd-division/country:us/state:vt/sldl:windsor-3"),
        District("Windsor-4", "lower", "ocd-division/country:us/state:vt/sldl:windsor-4"),
        District("Windsor-6-1", "lower", "ocd-division/country:us/state:vt/sldl:windsor-6-1"),
        District("Windsor-6-2", "lower", "ocd-division/country:us/state:vt/sldl:windsor-6-2"),
        # These Districts were Retired in 2022
        District("Bennington-2-1", "lower", "ocd-division/country:us/state:vt/sldl:bennington-2-1", 2),
        District("Bennington-2-2", "lower", "ocd-division/country:us/state:vt/sldl:bennington-2-2", 2),
        District("Caledonia-4", "lower", "ocd-division/country:us/state:vt/sldl:caledonia-4", 2),
        District("Chittenden-4-1", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-4-1"),
        District("Chittenden-4-2", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-4-2"),
        District("Chittenden-5-1", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-5-1"),
        District("Chittenden-5-2", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-5-2"),
        District("Chittenden-6-1", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-6-1", 2),
        District("Chittenden-6-2", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-6-2"),
        District("Chittenden-6-3", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-6-3", 2),
        District("Chittenden-6-4", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-6-4", 2),
        District("Chittenden-6-5", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-6-5", 2),
        District("Chittenden-6-6", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-6-6"),
        District("Chittenden-6-7", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-6-7", 2),
        District("Chittenden-7-1", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-7-1"),
        District("Chittenden-7-2", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-7-2"),
        District("Chittenden-7-3", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-7-3"),
        District("Chittenden-7-4", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-7-4"),
        District("Chittenden-8-1", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-8-1", 2),
        District("Chittenden-8-2", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-8-2", 2),
        District("Chittenden-8-3", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-8-3"),
        District("Chittenden-9-1", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-9-1", 2),
        District("Chittenden-9-2", "lower", "ocd-division/country:us/state:vt/sldl:chittenden-9-2", 2),
        District("Essex-Caledonia-Orleans", "lower", "ocd-division/country:us/state:vt/sldl:essex-caledonia-orleans"),
        District("Franklin-3-1", "lower", "ocd-division/country:us/state:vt/sldl:franklin-3-1", 2),
        District("Franklin-3-2", "lower", "ocd-division/country:us/state:vt/sldl:franklin-3-2"),
        District("Orleans-Caledonia", "lower", "ocd-division/country:us/state:vt/sldl:orleans-caledonia", 2),
        District("Rutland-5-1", "lower", "ocd-division/country:us/state:vt/sldl:rutland-5-1"),
        District("Rutland-5-2", "lower", "ocd-division/country:us/state:vt/sldl:rutland-5-2"),
        District("Rutland-5-3", "lower", "ocd-division/country:us/state:vt/sldl:rutland-5-3"),
        District("Rutland-5-4", "lower", "ocd-division/country:us/state:vt/sldl:rutland-5-4"),
        District("Rutland-Windsor-1", "lower", "ocd-division/country:us/state:vt/sldl:rutland-windsor-1"),
        District("Rutland-Windsor-2", "lower", "ocd-division/country:us/state:vt/sldl:rutland-windsor-2"),
        District("Washington-7", "lower", "ocd-division/country:us/state:vt/sldl:washington-7", 2),
        District("Windham-2-1", "lower", "ocd-division/country:us/state:vt/sldl:windham-2-1"),
        District("Windham-2-2", "lower", "ocd-division/country:us/state:vt/sldl:windham-2-2"),
        District("Windham-2-3", "lower", "ocd-division/country:us/state:vt/sldl:windham-2-3"),
        District("Windham-Bennington", "lower", "ocd-division/country:us/state:vt/sldl:windham-bennington"),
        District("Windham-Bennington-Windsor", "lower", "ocd-division/country:us/state:vt/sldl:windham-bennington-windsor"),
        District("Windsor-3-1", "lower", "ocd-division/country:us/state:vt/sldl:windsor-3-1"),
        District("Windsor-3-2", "lower", "ocd-division/country:us/state:vt/sldl:windsor-3-2", 2),
        District("Windsor-4-1", "lower", "ocd-division/country:us/state:vt/sldl:windsor-4-1"),
        District("Windsor-4-2", "lower", "ocd-division/country:us/state:vt/sldl:windsor-4-2", 2),
        District("Windsor-Rutland", "lower", "ocd-division/country:us/state:vt/sldl:windsor-rutland"),
        District("Chittenden", "upper", "ocd-division/country:us/state:vt/sldu:chittenden", 6),
        District("Essex-Orleans", "upper", "ocd-division/country:us/state:vt/sldu:essex-orleans", 2),
    ],
    "us": [
        District("CA-53", "lower", division_id=None),
        District("IL-18", "lower", division_id=None),
        District("IL-20", "lower", division_id=None),
        District("MA-10", "lower", division_id=None),
        District("MI-14", "lower", division_id=None),
        District("MO-9", "lower", division_id=None),
        District("NJ-13", "lower", division_id=None),
        District("NY-29", "lower", division_id=None),
        District("NY-27", "lower", division_id=None),
        District("OH-16", "lower", division_id=None),
        District("OH-17", "lower", division_id=None),
        District("OH-18", "lower", division_id=None),
        District("OK-6", "lower", division_id=None),
        District("PA-18", "lower", division_id=None),
        District("WV-3", "lower", division_id=None),
    ],
}



# Generated by Django 3.2.14 on 2024-07-05 18:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0044_bill_citations'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eventdocument',
            name='note',
            field=models.TextField(),
        ),
        migrations.AddIndex(
            model_name='voteevent',
            index=models.Index(fields=['dedupe_key'], name='opencivicdata_voteevent_dedupe__75a90b_idx'),
        ),
    ]



import typing
from enum import Enum
from pydantic import validator, root_validator
from .common import (
    BaseModel,
    Link,
    OtherName,
    validate_ocd_person,
    validate_ocd_jurisdiction,
    validate_ocd_organization,
    validate_str_no_newline,
)


class CommitteeChamber(str, Enum):
    UPPER = "upper"
    LOWER = "lower"
    LEGISLATURE = "legislature"


class CommitteeType(str, Enum):
    COMMITTEE = "committee"
    SUBCOMMITTEE = "subcommittee"


class Membership(BaseModel):
    name: str
    role: str
    person_id: typing.Optional[str] = None

    _validate_person_id = validator("person_id", allow_reuse=True)(validate_ocd_person)
    _validate_name = validator("name", allow_reuse=True)(validate_str_no_newline)
    _validate_role = validator("role", allow_reuse=True)(validate_str_no_newline)

    @validator("name")
    def not_blank(cls, val: str) -> str:
        val = val.strip()
        if not val:
            raise ValueError("empty name")
        return val


class ScrapeCommittee(BaseModel):
    name: str
    chamber: CommitteeChamber
    classification: CommitteeType = CommitteeType.COMMITTEE
    parent: typing.Optional[str] = None
    sources: typing.List[Link] = []
    links: typing.List[Link] = []
    other_names: typing.List[OtherName] = []
    members: typing.List[Membership] = []
    extras: dict = {}

    _validate_strs = validator("name", allow_reuse=True)(validate_str_no_newline)

    @root_validator
    def validate_parent_and_classification(
        cls, values: dict[str, typing.Any]
    ) -> dict[str, typing.Any]:
        if values.get("classification") == "subcommittee" and not values.get("parent"):
            raise ValueError("subcommittees must have a parent")
        if values.get("classification") == "committee" and values.get("parent"):
            raise ValueError(
                "committees may not have a parent, set classification=subcommittee"
            )
        return values

    def add_member(self, name: str, role: str = "member") -> None:
        self.members.append(Membership(name=name, role=role))

    def add_link(self, url: str, note: str = "") -> None:
        self.links.append(Link(url=url, note=note))

    def add_source(self, url: str, note: str = "") -> None:
        self.sources.append(Link(url=url, note=note))


class Committee(ScrapeCommittee):
    id: str
    jurisdiction: str

    _validate_jurisdiction = validator("jurisdiction", allow_reuse=True)(
        validate_ocd_jurisdiction
    )
    _validate_id = validator("id", allow_reuse=True)(validate_ocd_organization)
    _validate_parent = validator("parent", allow_reuse=True)(validate_ocd_organization)

    @root_validator
    def validate_has_members(cls, data: dict[str, typing.Any]) -> dict[str, typing.Any]:
        if not len(data.get("members", [])):  # type: ignore
            raise ValueError("committees must have members")
        return data

    def to_dict(self) -> dict[str, typing.Any]:
        # hack to always have id on top & always include classification
        return {
            "id": self.id,
            "jurisdiction": self.jurisdiction,
            "classification": self.classification.value
            if isinstance(self.classification, CommitteeType)
            else self.classification,
            **super().dict(exclude_defaults=True),
        }



# Generated by Django 3.2.2 on 2021-09-14 15:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "data",
            "0037_importobjects_runplan_scrapeobjects_scrapereport_sessiondataqualityreport",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="event",
            name="deleted",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="event",
            name="upstream_id",
            field=models.CharField(blank=True, max_length=300),
        ),
    ]



from .base import BaseImporter
from ._types import _JsonDict, Model, _RelatedModels
from ..data.models import Jurisdiction, LegislativeSession


class JurisdictionImporter(BaseImporter):
    _type = "jurisdiction"
    model_class = Jurisdiction
    related_models: _RelatedModels = {
        "legislative_sessions": (LegislativeSession, "jurisdiction_id", {})
    }
    merge_related = {"legislative_sessions": ["identifier"]}

    def get_object(self, data: _JsonDict) -> Model:
        return self.model_class.objects.get(
            division_id=data["division_id"], classification=data["classification"]
        )

    def prepare_for_db(self, data: _JsonDict) -> _JsonDict:
        for s in data["legislative_sessions"]:
            s.pop("_scraped_name", None)
        return data



from ..models import State, Chamber, District

VT = State(
    name="Vermont",
    abbr="VT",
    capital="Montpelier",
    capital_tz="America/New_York",
    fips="50",
    unicameral=False,
    legislature_name="Vermont General Assembly",
    legislature_organization_id="ocd-organization/3f26dc84-921f-48db-a9c9-ff843594ea85",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/300018cf-7c78-415b-8a3c-b00f1a0eb2fa",
    division_id="ocd-division/country:us/state:vt",
    jurisdiction_id="ocd-jurisdiction/country:us/state:vt/government",
    url="http://legislature.vermont.gov/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/1bba67ba-1a0c-4edc-a9de-32c2e5c13c6e",
        num_seats=150,
        title="Representative",
        districts=[
            District(
                "Addison-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:addison-1",
                2,
            ),
            District(
                "Addison-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:addison-2",
                1,
            ),
            District(
                "Addison-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:addison-3",
                2,
            ),
            District(
                "Addison-4",
                "lower",
                "ocd-division/country:us/state:vt/sldl:addison-4",
                2,
            ),
            District(
                "Addison-5",
                "lower",
                "ocd-division/country:us/state:vt/sldl:addison-5",
                1,
            ),
            District(
                "Addison-Rutland",
                "lower",
                "ocd-division/country:us/state:vt/sldl:addison-rutland",
                1,
            ),
            District(
                "Bennington-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:bennington-1",
                1,
            ),
            District(
                "Bennington-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:bennington-2",
                2,
            ),
            District(
                "Bennington-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:bennington-3",
                1,
            ),
            District(
                "Bennington-4",
                "lower",
                "ocd-division/country:us/state:vt/sldl:bennington-4",
                2,
            ),
            District(
                "Bennington-5",
                "lower",
                "ocd-division/country:us/state:vt/sldl:bennington-5",
                2,
            ),
            District(
                "Bennington-Rutland",
                "lower",
                "ocd-division/country:us/state:vt/sldl:bennington-rutland",
                1,
            ),
            District(
                "Caledonia-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:caledonia-1",
                1,
            ),
            District(
                "Caledonia-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:caledonia-2",
                1,
            ),
            District(
                "Caledonia-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:caledonia-3",
                2,
            ),
            District(
                "Caledonia-Washington",
                "lower",
                "ocd-division/country:us/state:vt/sldl:caledonia-washington",
                1,
            ),
            District(
                "Caledonia-Essex",
                "lower",
                "ocd-division/country:us/state:vt/sldl:caledonia-essex",
                2,
            ),
            District(
                "Chittenden-19",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-19",
                2,
            ),
            District(
                "Chittenden-10",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-10",
                1,
            ),
            District(
                "Chittenden-4",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-4",
                1,
            ),
            District(
                "Chittenden-5",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-5",
                1,
            ),
            District(
                "Chittenden-13",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-13",
                2,
            ),
            District(
                "Chittenden-14",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-14",
                2,
            ),
            District(
                "Chittenden-15",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-15",
                2,
            ),
            District(
                "Chittenden-16",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-16",
                2,
            ),
            District(
                "Chittenden-17",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-17",
                1,
            ),
            District(
                "Windsor-Addison",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-addison",
                1,
            ),
            District(
                "Washington-Orange",
                "lower",
                "ocd-division/country:us/state:vt/sldl:washington-orange",
                2,
            ),
            District(
                "Chittenden-Franklin",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-franklin",
                2,
            ),
            District(
                "Chittenden-8",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-8",
                1,
            ),
            District(
                "Chittenden-9",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-9",
                1,
            ),
            District(
                "Chittenden-12",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-12",
                1,
            ),
            District(
                "Chittenden-11",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-11",
                1,
            ),
            District(
                "Chittenden-18",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-18",
                2,
            ),
            District(
                "Chittenden-24",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-24",
                1,
            ),
            District(
                "Chittenden-23",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-23",
                2,
            ),
            District(
                "Chittenden-22",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-22",
                2,
            ),
            District(
                "Chittenden-21",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-21",
                2,
            ),
            District(
                "Chittenden-20",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-20",
                2,
            ),
            District(
                "Chittenden-25",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-25",
                1,
            ),
            District(
                "Chittenden-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-1",
                1,
            ),
            District(
                "Chittenden-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-2",
                2,
            ),
            District(
                "Chittenden-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-3",
                2,
            ),
            District(
                "Chittenden-6",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-6",
                1,
            ),
            District(
                "Chittenden-7",
                "lower",
                "ocd-division/country:us/state:vt/sldl:chittenden-7",
                1,
            ),
            District(
                "Essex-Caledonia",
                "lower",
                "ocd-division/country:us/state:vt/sldl:essex-caledonia",
                1,
            ),
            District(
                "Essex-Orleans",
                "lower",
                "ocd-division/country:us/state:vt/sldl:essex-orleans",
                1,
            ),
            District(
                "Franklin-8",
                "lower",
                "ocd-division/country:us/state:vt/sldl:franklin-8",
                1,
            ),
            District(
                "Franklin-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:franklin-1",
                2,
            ),
            District(
                "Franklin-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:franklin-2",
                1,
            ),
            District(
                "Franklin-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:franklin-3",
                1,
            ),
            District(
                "Franklin-4",
                "lower",
                "ocd-division/country:us/state:vt/sldl:franklin-4",
                2,
            ),
            District(
                "Franklin-5",
                "lower",
                "ocd-division/country:us/state:vt/sldl:franklin-5",
                2,
            ),
            District(
                "Franklin-6",
                "lower",
                "ocd-division/country:us/state:vt/sldl:franklin-6",
                1,
            ),
            District(
                "Franklin-7",
                "lower",
                "ocd-division/country:us/state:vt/sldl:franklin-7",
                1,
            ),
            District(
                "Grand Isle-Chittenden",
                "lower",
                "ocd-division/country:us/state:vt/sldl:grand_isle-chittenden",
                2,
            ),
            District(
                "Lamoille-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:lamoille-1",
                1,
            ),
            District(
                "Lamoille-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:lamoille-2",
                2,
            ),
            District(
                "Lamoille-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:lamoille-3",
                1,
            ),
            District(
                "Lamoille-Washington",
                "lower",
                "ocd-division/country:us/state:vt/sldl:lamoille-washington",
                2,
            ),
            District(
                "Orange-1", "lower", "ocd-division/country:us/state:vt/sldl:orange-1", 1
            ),
            District(
                "Orange-2", "lower", "ocd-division/country:us/state:vt/sldl:orange-2", 1
            ),
            District(
                "Orange-3", "lower", "ocd-division/country:us/state:vt/sldl:orange-3", 1
            ),
            District(
                "Orange-Caledonia",
                "lower",
                "ocd-division/country:us/state:vt/sldl:orange-caledonia",
                1,
            ),
            District(
                "Orange-Washington-Addison",
                "lower",
                "ocd-division/country:us/state:vt/sldl:orange-washington-addison",
                2,
            ),
            District(
                "Orleans-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:orleans-1",
                1,
            ),
            District(
                "Orleans-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:orleans-2",
                1,
            ),
            District(
                "Orleans-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:orleans-3",
                1,
            ),
            District(
                "Orleans-4",
                "lower",
                "ocd-division/country:us/state:vt/sldl:orleans-4",
                1,
            ),
            District(
                "Orleans-Lamoille",
                "lower",
                "ocd-division/country:us/state:vt/sldl:orleans-lamoille",
                2,
            ),
            District(
                "Rutland-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-1",
                1,
            ),
            District(
                "Rutland-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-2",
                2,
            ),
            District(
                "Rutland-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-3",
                1,
            ),
            District(
                "Rutland-4",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-4",
                1,
            ),
            District(
                "Rutland-5",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-5",
                1,
            ),
            District(
                "Rutland-6",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-6",
                1,
            ),
            District(
                "Rutland-Bennington",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-bennington",
                1,
            ),
            District(
                "Rutland-Windsor",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-windsor",
                1,
            ),
            District(
                "Windsor-Windham",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-windham",
                1,
            ),
            District(
                "Washington-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:washington-1",
                2,
            ),
            District(
                "Washington-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:washington-2",
                2,
            ),
            District(
                "Washington-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:washington-3",
                2,
            ),
            District(
                "Washington-4",
                "lower",
                "ocd-division/country:us/state:vt/sldl:washington-4",
                2,
            ),
            District(
                "Washington-5",
                "lower",
                "ocd-division/country:us/state:vt/sldl:washington-5",
                1,
            ),
            District(
                "Washington-6",
                "lower",
                "ocd-division/country:us/state:vt/sldl:washington-6",
                1,
            ),
            District(
                "Washington-Chittenden",
                "lower",
                "ocd-division/country:us/state:vt/sldl:washington-chittenden",
                2,
            ),
            District(
                "Windham-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-1",
                1,
            ),
            District(
                "Windham-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-2",
                1,
            ),
            District(
                "Windham-9",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-9",
                1,
            ),
            District(
                "Windham-8",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-8",
                1,
            ),
            District(
                "Windham-7",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-7",
                1,
            ),
            District(
                "Windham-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-3",
                2,
            ),
            District(
                "Windham-4",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-4",
                1,
            ),
            District(
                "Windham-5",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-5",
                1,
            ),
            District(
                "Windham-6",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-6",
                1,
            ),
            District(
                "Windham-Windsor-Bennington",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windham-bennington-windsor",
                1,
            ),
            District(
                "Windsor-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-1",
                2,
            ),
            District(
                "Windsor-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-2",
                1,
            ),
            District(
                "Windsor-3",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-3",
                2,
            ),
            District(
                "Windsor-4",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-4",
                1,
            ),
            District(
                "Windsor-5",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-5",
                1,
            ),
            District(
                "Windsor-6",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-6",
                2,
            ),
            District(
                "Rutland-11",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-11",
                1,
            ),
            District(
                "Rutland-7",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-7",
                1,
            ),
            District(
                "Rutland-8",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-8",
                1,
            ),
            District(
                "Rutland-10",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-10",
                1,
            ),
            District(
                "Rutland-9",
                "lower",
                "ocd-division/country:us/state:vt/sldl:rutland-9",
                1,
            ),
            District(
                "Windsor-Orange-1",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-orange-1",
                1,
            ),
            District(
                "Windsor-Orange-2",
                "lower",
                "ocd-division/country:us/state:vt/sldl:windsor-orange-2",
                2,
            ),
        ],
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/21504998-e5c7-44b4-8ebe-99fe57ef520c",
        num_seats=30,
        title="Senator",
        districts=[
            District(
                "Addison", "upper", "ocd-division/country:us/state:vt/sldu:addison", 2
            ),
            District(
                "Bennington",
                "upper",
                "ocd-division/country:us/state:vt/sldu:bennington",
                2,
            ),
            District(
                "Caledonia",
                "upper",
                "ocd-division/country:us/state:vt/sldu:caledonia",
                1,
            ),
            District(
                "Chittenden Central",
                "upper",
                "ocd-division/country:us/state:vt/sldu:chittenden-central",
                3,
            ),
            District(
                "Chittenden North",
                "upper",
                "ocd-division/country:us/state:vt/sldu:chittenden-north",
                1,
            ),
            District(
                "Chittenden Southeast",
                "upper",
                "ocd-division/country:us/state:vt/sldu:chittenden-southeast",
                3,
            ),
            District(
                "Orleans", "upper", "ocd-division/country:us/state:vt/sldu:orleans", 1
            ),
            District(
                "Essex", "upper", "ocd-division/country:us/state:vt/sldu:essex", 1
            ),
            District(
                "Franklin", "upper", "ocd-division/country:us/state:vt/sldu:franklin", 2
            ),
            District(
                "Grand Isle",
                "upper",
                "ocd-division/country:us/state:vt/sldu:grand_isle",
                1,
            ),
            District(
                "Lamoille", "upper", "ocd-division/country:us/state:vt/sldu:lamoille", 1
            ),
            District(
                "Orange", "upper", "ocd-division/country:us/state:vt/sldu:orange", 1
            ),
            District(
                "Rutland", "upper", "ocd-division/country:us/state:vt/sldu:rutland", 3
            ),
            District(
                "Washington",
                "upper",
                "ocd-division/country:us/state:vt/sldu:washington",
                3,
            ),
            District(
                "Windham", "upper", "ocd-division/country:us/state:vt/sldu:windham", 2
            ),
            District(
                "Windsor", "upper", "ocd-division/country:us/state:vt/sldu:windsor", 3
            ),
        ],
    ),
)



import os

# django settings for tests
SECRET_KEY = "test"
INSTALLED_APPS = (
    "django.contrib.contenttypes",
    "openstates.data",
)
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": "testdb",
        "USER": os.environ.get("PGUSER", "openstates"),
        "PASSWORD": os.environ.get("PGPASS", "openstates"),
        "HOST": os.environ.get("PGHOST", "localhost"),
        "PORT": os.environ.get("PGPORT", 5432),
    }
}
MIDDLEWARE_CLASSES = ()



from ..models import State, Chamber, District

ZA = State(
    name="South Africa",
    abbr="ZA",
    capital="Cape Town",
    capital_tz="Africa/Johannesburg",
    fips="SF",
    unicameral=False,
    legislature_name="South Africa Parliament",
    legislature_organization_id="ocd-organization/de2b7383-ee98-4f01-bc99-650ba84358a4",
    executive_name="President of the South Africa",
    executive_organization_id="ocd-organization/2c4b6383-7560-4c15-8266-6fc6b717a8ef",
    division_id="ocd-division/country:za",
    jurisdiction_id="ocd-jurisdiction/country:za/government",
    url="https://www.parliament.gov.za/",
    lower=Chamber(
        chamber_type="lower",
        name="National Assembly",
        organization_id="ocd-organization/fac94935-991f-424f-b02a-8f1336a6e540",
        num_seats=400,
        title="Member",
        districts=[
            # made up ocd-divisions based on party abbreviations since there aren't official ones and the number of
            # seats are based on the number of voters that voted for each party in the elections
            # https://www.parliament.gov.za/national-assembly
            District(
                "African National Congress",
                "lower",
                "ocd-division/country:za/party:anc",
                230,
            ),
            District(
                "Democratic Alliance",
                "lower",
                "ocd-division/country:za/party:da",
                84,
            ),
            District(
                "Economic Freedom Fighters",
                "lower",
                "ocd-division/country:za/party:dff",
                44,
            ),
            District(
                "Inkatha Freedom Party",
                "lower",
                "ocd-division/country:za/party:ifp",
                14,
            ),
            District(
                "Freedom Front Plus",
                "lower",
                "ocd-division/country:za/party:ff-plus",
                10,
            ),
            District(
                "African Christian Democratic Party",
                "lower",
                "ocd-division/country:za/party:acdp",
                4,
            ),
            District(
                "United Democratic Movement",
                "lower",
                "ocd-division/country:za/party:udm",
                2,
            ),
            District(
                "African Transformation Movement",
                "lower",
                "ocd-division/country:za/party:atm",
                2,
            ),
            District(
                "Good Party",
                "lower",
                "ocd-division/country:za/party:good",
                2,
            ),
            District(
                "National Freedom Party",
                "lower",
                "ocd-division/country:za/party:ntp",
                2,
            ),
            District(
                "African Independent Congress",
                "lower",
                "ocd-division/country:za/party:aic",
                2,
            ),
            District(
                "Congress of the People",
                "lower",
                "ocd-division/country:za/party:cope",
                2,
            ),
            District(
                "Pan Africanist Congress",
                "lower",
                "ocd-division/country:za/party:pac",
                1,
            ),
            District(
                "Al Jama-ah",
                "lower",
                "ocd-division/country:za/party:aljama-ah",
                1,
            ),
        ],
    ),
    upper=Chamber(
        chamber_type="upper",
        name="National Council of Provinces",
        organization_id="ocd-organization/33ec4697-6a77-4edb-a279-1f8df6509469",
        num_seats=90,
        title="Member",
        districts=[
            # ocd-divisions found here
            # https://github.com/opencivicdata/ocd-division-ids/blob/master/identifiers/country-za/provinces.csv
            District(
                "Eastern Cape",
                "upper",
                "ocd-division/country:za/province:ec",
                10,
            ),
            District(
                "Free State",
                "upper",
                "ocd-division/country:za/province:fs",
                10,
            ),
            District(
                "Gauteng",
                "upper",
                "ocd-division/country:za/province:gp",
                10,
            ),
            District(
                "KwaZulu-Natal",
                "upper",
                "ocd-division/country:za/province:kzn",
                10,
            ),
            District(
                "Limpopo",
                "upper",
                "ocd-division/country:za/province:lp",
                10,
            ),
            District(
                "Mpumalanga",
                "upper",
                "ocd-division/country:za/province:mp",
                10,
            ),
            District(
                "Northern Cape",
                "upper",
                "ocd-division/country:za/province:nc",
                10,
            ),
            District(
                "North West",
                "upper",
                "ocd-division/country:za/province:nw",
                10,
            ),
            District(
                "Western Cape",
                "upper",
                "ocd-division/country:za/province:wc",
                10,
            ),
        ]
    ),
)
